/*
              Program Name : EDIT
              Package Name : LB
              Purpose      : String editing function
              Institution  : Conroy & Conroy
              Date Written : 30-Oct-87
              Written By   : Alan Conroy
              Version      : 1.0

	      Copyright (C) 1987 by Alan Conroy.  Released to the public domain.

              TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *     DATE      BY             REASON                   *
        *                                                       *
        *                                                       *
        *********************************************************

        *********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

          This function will perform various editing functions on
        strings.  The format for function usage is: edit( dst, src, flgs )
        where flgs has the following meanings:

              AND     1 = Trim Most Significant Bit.
              AND     2 = Remove all spaces and tabs.
              AND     4 = Remove all nulls, linefeeds, formfeeds, carriage
                          returns, and deletes.
              AND     8 = Remove leading spaces.
              AND    16 = Reduce spaces and tabs to one space.
              AND    32 = Convert lower to upper case.
              AND    64 = Convert '[' to '(' and ']' to ')'.
              AND   128 = Remove all trailing spaces.
              AND   256 = Leave characters within quotes (") unmodified.
              AND   512 = Convert upper case to lower case.
              AND  1024 = Convert '[' to '{' and ']' to '}'.
              AND  2048 = Convert '(' to '[' and ')' to ']'.
              AND  4096 = Convert '(' to '{' and ')' to '}'.
              AND  8192 = Convert '{' to '(' and '}' to ')'.
              AND 16384 = Convert '{' to '[' and '}' to ']'.
*/

#include <string.h>

#include "clib.h"

void Edit( char *Dst, char *Src, int Flags )
{
    int		Leading, Quote, Quote_Type ;
    int		OK ; /* True if character gets moved to Dst */
    int		Space ;
    char	Chr, Result ; /* Current character */
    char	*Last ; /* Pointer to last non-space character */
    char	*SI, *DI ; /* Source and Destination index pointers */


    /* Validation */

    if ( strlen( Src ) == 0 ) /* No edits on zero-length strings */
    {
        strcpy( Dst, "" ) ;
        return ;
    };

    /* Disallow lower-case command if upper-case command is also present */
    if ( ( Flags & 544 ) == 544 )
    {
	Flags = Flags & ( ~ 512 ) ;
    };

    /* Disallow [] to {} if [] to () */
    if ( ( Flags & 1088 ) == 1088 )
    {
	Flags = Flags & ( ~ 1024 ) ;
    };

    /* Disallow () to {} if () to [] */
    if ( ( Flags & 6144 ) == 6144 )
    {
	Flags = Flags & ( ~ 4096 ) ;
    };

    /* Disallow {} to [] if {} to () */
    if ( ( Flags & 24576 ) == 24576 )
    {
	Flags = Flags & ( ~ 16384 ) ;
    };


    /* Set up */

    SI = Src ; /* Address of source string */
    DI = Dst ; /* Address of destination string */
    Quote = 0 ;
    Space = False ; /* No spaces */
    Last = Dst - 1 ;
    Leading = True ; /* Assume leading spaces until proven otherwise */


    /* Do actual editing */

    while ( *SI != 0 ) /* Until we hit the end of the string */
    {
	Result = *SI++ ; /* Get next character */
	Chr = Result & 127; /* Trim high bit for easy comparisons */
	OK = True ; /* This byte is OK so far */
	if ( Quote == 0 ) /* Not within quotes (only set if flags & 256) */
	{
	    if ( ( Chr != 9 ) && ( Chr != 32 ) ) /* Not a tab or space */
	    {
		Leading = False ; /* End of leading spaces */
	    };
	    if ( ( Flags & 1 ) == 1 ) /* Trim parity bit */
	    {
		Result = Chr ;
	    };
	    if ( ( Flags & 2 ) == 2 ) /* Remove all spaces/tabs? */
	    {
		if ( ( Chr == 32 ) || ( Chr == 9 ) ) /* Space or tab */
		{
		    OK = False ;
		};
	    };
	    if ( ( Flags & 4 ) == 4 )
	    {
		if ( ( Chr == 10 ) || ( Chr == 12 ) || ( Chr == 13 ) || ( Chr == 127 ) )
		/* Ignore LF, FF, CR, and ESC */
		{
		    OK = False ;
		};
	    };
	    if ( ( Flags & 8 ) == 8 ) /* Ignore leading spaces */
	    {
		if ( Leading ) /* Currently have leading spaces */
		{
		    OK = False ;
		};
	    };
	    if ( ( Flags & 16 ) == 16 ) /* Reduce tabs/spaces to singe space */
	    {
		if ( ( Chr == 9 ) || ( Chr == 32 ) )
		{
		    if ( Space )
		    {
			OK = False;
		    }
		    else
		    {
			Result = 32 ;
			Space = True ;
		    };
		};
	    };
	    if ( ( Flags & 32 ) == 32 ) /* Lower to upper case */
	    {
		if ( ( Chr >= 'a' ) && ( Chr <= 'z' ) )
	 	{
		    Result = Result - 32; /* Convert to uppercase */
		};
	    };
	    if ( ( Flags & 512 ) == 512 ) /* Upper to lower case */
	    {
		if ( ( Chr >= 'A' ) && ( Chr <= 'Z' ) )
		{
		    Result = Result + 32 ; /* Convert to lowercase */
		};
	    };
	    if ( ( Flags & 64 ) == 64 ) /* Convert [] to () */
	    {
		if ( Chr == '[' )
		{
		    Result = '(' ;
		}
		else
		{
		    if ( Chr == ']' )
		    {
			Result = ')' ;
		    };
		};
	    };
	    if ( ( Flags & 1024 ) == 1024 ) /* Convert [] to {} */
	    {
		if ( Chr == '[' )
		{
		    Result = '{' ;
		}
		else
		{
		    if ( Chr == ']' )
		    {
			Result = '}' ;
		    };
		};
	    };
	    if ( ( Flags & 2048 ) == 2048 ) /* Convert () to [] */
	    {
		if ( Chr == '(' )
		{
		    Result = '[' ;
		}
		else
		{
		    if ( Chr == ')' )
		    {
			Result = ']' ;
		    };
		};
	    };
	    if ( ( Flags & 4096 ) == 4096 ) /* Convert () to {} */
	    {
		if ( Chr == '(' )
		{
		    Result = '{' ;
		}
		else
		{
		    if ( Chr == ')' )
		    {
			Result = '}' ;
		    };
		};
	    };
	    if ( ( Flags & 8192 ) == 8192 ) /* Convert {} to () */
	    {
	 	if ( Chr == '{' )
	 	{
		    Result = '(' ;
		}
		else
		{
		    if ( Chr == '}' )
		    {
			Result = ')' ;
		    };
		};
	    };
	    if ( ( Flags & 16384 ) == 16384 ) /* Convert {} to [] */
	    {
		if ( Chr == '{' )
		{
		    Result = '[' ;
		}
		else
		{
		    if ( Chr == '}' )
		    {
			Result = ']' ;
		    };
		};
	    };
	};
	if ( OK )
	{
	    *DI++ = Result ; /* Build result */
	    if ( ( Chr == 9 ) || ( Chr == 32 ) ) /* Tab or space */
	    {
		Space = True ;
	    };
	};
	if ( ( Chr != 9 ) && ( Chr != 32 ) ) /* Not tab or space */
	{
	    Space = False ;
	    if ( ( Flags & 256 ) == 256 ) /* Leave quoted text alone */
	    {
		if ( ( Chr == 34 ) || ( Chr == 39 ) )
		{
		    if ( !Quote ) /* Not within quotes */
		    {
			Quote_Type = Chr ; /* New quote type */
			Quote = True ; /* Now inside quotes */
		    }
		    else
		    {
			if ( Quote_Type == Chr ) /* Ending quote */
			{
			    Quote = False ; /* End of quote */
			};
		    };
		};
	    };
	    Last = DI - 1 ; /* Last non-space character */
	};
    };


    /* Clean up */

    if ( ( Flags & 128 ) == 128 ) /* Trim trailing spaces */
    {
	if ( !Quote ) /* Not within quoted segment */
	{
	    DI = Last + 1 ; /* Reset to last non-space */
	};
    };
    *DI = 0 ; /* Null terminator */
};
